@@ -22,17 +22,76 @@
 
 #include <config.h>
 
-#include <stdio.h>
+#include <string.h>
 #include <gtk/gtk.h>
-#include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
 
 #include "gui.h"
+#include "gui_utils.h"
 #include "prefs.h"
-#include "main.h"
 #include "i18n.h"
 
 
+/*--------------------------------------------------------------------*/
+
+void
+about_window_close_cb (GtkWidget *widget, GdkEvent *event, gpointer user_data) {
+
+    GUI *appGUI = (GUI *)user_data;
+
+    gdk_window_get_root_origin ((appGUI->about_window)->window,
+                                &config.about_window_x, &config.about_window_y);
+
+    gui_url_remove_links (&appGUI->about_links_list, &appGUI->about_link_index);
+
+    gtk_widget_destroy (appGUI->about_window);
+}
+
+/*--------------------------------------------------------------------*/
+
+gint
+about_key_press_cb (GtkWidget *widget, GdkEventKey *event, gpointer user_data) {
+
+    GUI *appGUI = (GUI *)user_data;
+
+    if (event->keyval == GDK_Escape) {
+        about_window_close_cb (NULL, NULL, appGUI);
+        return TRUE;
+    }
+
+    return FALSE;
+}
+
+/*--------------------------------------------------------------------*/
+
+void
+about_close_button_cb (GtkWidget *widget, gpointer user_data) {
+
+    GUI *appGUI = (GUI *)user_data;
+
+    about_window_close_cb (NULL, NULL, appGUI);
+}
+
+/*--------------------------------------------------------------------*/
+
+void
+about_create_window (GUI *appGUI) {
+
+GtkWidget       *vbox1;
+GtkWidget       *vbox2;
+GtkWidget       *hseparator;
+GtkWidget       *hbuttonbox;
+GtkWidget       *close_button;
+GtkWidget       *notebook;
+GtkWidget       *label;
+GtkWidget       *scrolled_window;
+GtkWidget       *viewport;
+GtkTextBuffer   *entry_buffer = NULL;
+GtkWidget       *text_sheet = NULL;
+GtkTextIter     iter;
+gchar           buffer[BUFFER_SIZE];
+gint            i;
+
 gchar text_contributors[] = {
     "                Vadim Berezniker\n"
     "                Yann Golanski\n"
@@ -48,11 +107,6 @@ gchar text_contributors[] = {
     "                Marianne Ibbotson\n"
 };
 
-gchar text_translators[] = {
-    "                [pl] Piotr Mąka <sill@ll.pl>\n"
-    "                [ru] Taci Taclipoka <taclipoka@gmail.com>\n"
-};
-
 gchar text_postcards_address[] = {
     "                   Piotrek\n"
     "                   ul. Ku Słońcu 90\n"
@@ -68,6 +122,15 @@ gchar text_received_postcards[] = {
     "  * Wolfram from Kyoto (Japan)\n"
     "  * Gabriel from Marseille (France)\n"
     "  * Unknown from Okinawa Islands (Japan)\n"
+    "  * David from Madrit (Spain)\n"
+    "  * Raúl from Collado Villalba in Madrit (Spain)\n"
+    "  * Phil from Montlingen (Switzerland)\n"
+    "  * Ken from Boston in Massachusetts (USA)\n"
+    "  * Jerry from Wichita in Kansas (USA)\n"
+    "  * Chris from Mâcon (France)\n"
+    "  * Giulia and Daniele from Genova (Italy)\n"
+    "  * Csaba, Brendon and Xenia from Budapest (Hungary)\n"
+    "  * Kate from Upholland (UK)\n"
 };
 
 gchar text_license[] = {
@@ -86,70 +149,37 @@ gchar text_license[] = {
     "Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA."
 };
 
+gchar *translators[] = {
+    "de",   "Mathias Brodala",      "info@noctus.net",
+    "",     "Frank Polte",          "frederyk@gmx.de",
+    "fr",   "Adrian Courrèges",     "a.courreges@gmail.com",
+    "hu",   "Szigetvári Csaba",     "csaba.szigetvari@informedia.hu",
+    "pl",   "Piotr Mąka",           "sill@ll.pl",
+    "pt",   "Bruno Ramos",          "ladrilho@gmail.com",
+    "ru",   "Taci Taclipoka",       "taclipoka@gmail.com"
+};
 
-GtkWidget       *about_window;
+    appGUI->about_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+    gtk_window_set_transient_for (GTK_WINDOW(appGUI->about_window),GTK_WINDOW(appGUI->main_window));
+    gtk_window_set_resizable (GTK_WINDOW (appGUI->about_window), TRUE);
+    gtk_window_set_modal (GTK_WINDOW(appGUI->about_window), TRUE);
+    gtk_container_set_border_width (GTK_CONTAINER (appGUI->about_window), 12);
+    gtk_window_set_title (GTK_WINDOW (appGUI->about_window), _("About"));
+    gtk_widget_set_size_request (appGUI->about_window, 470, 550);
 
-extern GtkWidget *main_window;
+    g_signal_connect (G_OBJECT (appGUI->about_window), "delete_event",
+                        G_CALLBACK(about_window_close_cb), appGUI);
+    g_signal_connect (G_OBJECT(appGUI->about_window), "key_press_event",
+                        G_CALLBACK(about_key_press_cb), appGUI);
 
-
-void
-about_window_close_cb (void) {
-
-    gdk_window_get_root_origin (about_window->window,
-                                &config.about_window_x, &config.about_window_y);
-
-    gtk_widget_destroy (about_window);
-}
-
-gint
-about_key_press_cb (GtkWidget *widget, GdkEventKey *event, gpointer data) {
-
-    if (event->keyval == GDK_Escape) {
-        about_window_close_cb ();
-        return TRUE;
-    }
-
-    return FALSE;
-}
-
-void
-about_create_window (void) {
-
-GtkWidget       *vbox1;
-GtkWidget       *vbox2;
-GtkWidget       *hseparator;
-GtkWidget       *hbuttonbox;
-GtkWidget       *close_button;
-GtkWidget       *notebook;
-GtkWidget       *label;
-GtkWidget       *scrolled_window;
-GtkWidget       *viewport;
-GtkTextBuffer   *entry_buffer = NULL;
-GtkWidget       *text_sheet = NULL;
-GtkTextIter     iter;
-gchar           buffer[BUFFER_SIZE];
-
-    about_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-    gtk_window_set_transient_for(GTK_WINDOW(about_window),GTK_WINDOW(main_window));
-    gtk_window_set_resizable (GTK_WINDOW (about_window), TRUE);
-    gtk_window_set_modal(GTK_WINDOW(about_window), TRUE);
-    gtk_container_set_border_width (GTK_CONTAINER (about_window), 12);
-    gtk_window_set_title (GTK_WINDOW (about_window), _("About"));
-    gtk_widget_set_size_request (about_window, 410, 460);
-
-    g_signal_connect (G_OBJECT (about_window), "delete_event",
-                        G_CALLBACK(about_window_close_cb), NULL);
-    g_signal_connect (G_OBJECT(about_window), "key_press_event",
-                        G_CALLBACK(about_key_press_cb), NULL);
-
-    gtk_window_move (GTK_WINDOW (about_window),
+    gtk_window_move (GTK_WINDOW (appGUI->about_window),
                                 config.about_window_x, config.about_window_y);
 
-    gtk_widget_show (about_window);
+    gtk_widget_show (appGUI->about_window);
 
     vbox1 = gtk_vbox_new (FALSE, 0);
     gtk_widget_show (vbox1);
-    gtk_container_add (GTK_CONTAINER (about_window), vbox1);
+    gtk_container_add (GTK_CONTAINER (appGUI->about_window), vbox1);
 
     notebook = gtk_notebook_new ();
     gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
@@ -174,6 +204,7 @@ gchar           buffer[BUFFER_SIZE];
 
     entry_buffer = gtk_text_buffer_new (NULL);
     gtk_text_buffer_get_iter_at_offset (entry_buffer, &iter, 0);
+
     gtk_text_buffer_create_tag (entry_buffer, "bold",
                   "weight", PANGO_WEIGHT_ULTRABOLD, NULL);
     gtk_text_buffer_create_tag (entry_buffer, "big",
@@ -182,34 +213,74 @@ gchar           buffer[BUFFER_SIZE];
                   "style", PANGO_STYLE_ITALIC, NULL);
     gtk_text_buffer_create_tag (entry_buffer, "center",
                   "justification", GTK_JUSTIFY_CENTER, NULL);
-    gtk_text_buffer_get_iter_at_offset (entry_buffer, &iter, 0);
-    text_sheet = gtk_text_view_new_with_buffer (entry_buffer);
-    gtk_text_view_set_pixels_above_lines(GTK_TEXT_VIEW(text_sheet), 2);
-    gtk_text_view_set_left_margin(GTK_TEXT_VIEW(text_sheet), 8);
-    gtk_text_view_set_right_margin(GTK_TEXT_VIEW(text_sheet), 8);
-    gtk_text_view_set_editable(GTK_TEXT_VIEW (text_sheet), FALSE);
-    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW (text_sheet), FALSE);
-    gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (text_sheet), GTK_WRAP_WORD);
-    gtk_widget_show (text_sheet);
-    gtk_container_add (GTK_CONTAINER (viewport), text_sheet);
+    gtk_text_buffer_create_tag (entry_buffer, "fixed",
+			      "family", "monospace", NULL);
 
+    gtk_text_buffer_get_iter_at_offset (entry_buffer, &iter, 0);
+    appGUI->about_textview = gtk_text_view_new_with_buffer (entry_buffer);
+    gtk_text_view_set_pixels_above_lines (GTK_TEXT_VIEW(appGUI->about_textview), 2);
+    gtk_text_view_set_left_margin (GTK_TEXT_VIEW(appGUI->about_textview), 8);
+    gtk_text_view_set_right_margin (GTK_TEXT_VIEW(appGUI->about_textview), 8);
+    gtk_text_view_set_editable (GTK_TEXT_VIEW (appGUI->about_textview), FALSE);
+    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (appGUI->about_textview), FALSE);
+    gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (appGUI->about_textview), GTK_WRAP_WORD);
+    gtk_widget_show (appGUI->about_textview);
+    gtk_container_add (GTK_CONTAINER (viewport), appGUI->about_textview);
+
+    gui_url_setup (&appGUI->about_links_list, &appGUI->about_link_index, appGUI->about_textview, appGUI);
+
+    gtk_text_buffer_insert (entry_buffer, &iter, "\n", -1);
+
+#ifndef REV
+    sprintf (buffer, "Kanatest %s\n", VERSION);
+#else
+    sprintf (buffer, "Kanatest SVN r%d\n", REV);
+#endif
     gtk_text_buffer_insert_with_tags_by_name (entry_buffer, &iter,
-                        "\nKanatest " VERSION "\n", -1, "big", "center", NULL);
-    sprintf(buffer, "\n(%s %s, %s)\n\n", _("compiled on"), __DATE__, __TIME__);
+                        buffer, -1, "big", "center", NULL);
+
+    gtk_text_buffer_insert (entry_buffer, &iter, "\n", -1);
+    gui_url_insert_link(&appGUI->about_links_list, &appGUI->about_link_index, appGUI->about_textview,
+			&iter, NULL, 0, "http://clay.ll.pl/kanatest", TRUE, appGUI);
+    gtk_text_buffer_insert (entry_buffer, &iter, "\n", -1);
+    g_snprintf (buffer, BUFFER_SIZE, "\n(%s %s, %s)\n\n", _("compiled on"), __DATE__, __TIME__);
     gtk_text_buffer_insert_with_tags_by_name (entry_buffer, &iter, buffer, -1, "center", "italic", NULL);
 
-    sprintf(buffer, "\n%s ", _("Homepage:"));
+    g_snprintf (buffer, BUFFER_SIZE, "\n%s:\n", _("Programming"));
     gtk_text_buffer_insert_with_tags_by_name (entry_buffer, &iter, buffer, -1, "bold", NULL);
-    gtk_text_buffer_insert (entry_buffer, &iter, "http://clay.ll.pl/kanatest\n\n", -1);
+    gtk_text_buffer_insert (entry_buffer, &iter, "     Tomasz Mąka <", -1);
+    gui_url_insert_link (&appGUI->about_links_list, &appGUI->about_link_index, appGUI->about_textview,
+			             &iter, NULL, 0, "pasp@ll.pl", TRUE, appGUI);
+    gtk_text_buffer_insert (entry_buffer, &iter, ">\n", -1);
 
-    sprintf(buffer, "\n%s\n", _("Programming:"));
+    g_snprintf (buffer, BUFFER_SIZE, "\n%s:\n", _("Graphics"));
     gtk_text_buffer_insert_with_tags_by_name (entry_buffer, &iter, buffer, -1, "bold", NULL);
-    gtk_text_buffer_insert (entry_buffer, &iter, "     Tomasz Mąka <pasp@users.sourceforge.net>\n", -1);
+    gtk_text_buffer_insert (entry_buffer, &iter, "     Maja Kocoń (", -1);
+    gui_url_insert_link (&appGUI->about_links_list, &appGUI->about_link_index, appGUI->about_textview,
+			             &iter, NULL, 0, "http://ironya.ll.pl", TRUE, appGUI);
+    gtk_text_buffer_insert (entry_buffer, &iter, ")\n", -1);
+    gtk_text_buffer_insert (entry_buffer, &iter, "     Piotr Mąka <", -1);
+    gui_url_insert_link (&appGUI->about_links_list, &appGUI->about_link_index, appGUI->about_textview,
+			             &iter, NULL, 0, "sill@ll.pl", TRUE, appGUI);
+    gtk_text_buffer_insert (entry_buffer, &iter, ">\n", -1);
 
-    sprintf(buffer, "\n%s\n", _("Graphics:"));
+    g_snprintf (buffer, BUFFER_SIZE, "\n%s:\n", _("Translators"));
     gtk_text_buffer_insert_with_tags_by_name (entry_buffer, &iter, buffer, -1, "bold", NULL);
-    gtk_text_buffer_insert (entry_buffer, &iter, "     Piotr Mąka <sill@ll.pl>\n", -1);
-    gtk_text_buffer_insert (entry_buffer, &iter, "     Maja Kocoń (http://ironya.ll.pl)\n", -1);
+
+    for (i=0; i < (sizeof(translators) / sizeof(translators[0]) / 3); i++) {
+        gtk_text_buffer_insert (entry_buffer, &iter, "     ", -1);
+        if (strlen(translators[i*3+0])) {
+            g_snprintf (buffer, BUFFER_SIZE, "[%s]  ", translators[i*3+0]);      
+            gtk_text_buffer_insert_with_tags_by_name (entry_buffer, &iter, buffer, -1, "fixed", NULL);
+        } else {
+            gtk_text_buffer_insert_with_tags_by_name (entry_buffer, &iter, "      ", -1, "fixed", NULL);
+        }
+        g_snprintf (buffer, BUFFER_SIZE, "%s <", translators[i*3+1]);
+        gtk_text_buffer_insert (entry_buffer, &iter, buffer, -1);
+        gui_url_insert_link (&appGUI->about_links_list, &appGUI->about_link_index, appGUI->about_textview,
+                             &iter, NULL, 0, translators[i*3+2], TRUE, appGUI);
+        gtk_text_buffer_insert (entry_buffer, &iter, ">\n", -1);
+    }
 
     label = gtk_label_new (_("About"));
     gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox2, label);
@@ -238,25 +309,21 @@ gchar           buffer[BUFFER_SIZE];
                   "size", 12 * PANGO_SCALE, NULL);
     gtk_text_buffer_get_iter_at_offset (entry_buffer, &iter, 0);
     text_sheet = gtk_text_view_new_with_buffer (entry_buffer);
-    gtk_text_view_set_pixels_above_lines(GTK_TEXT_VIEW(text_sheet), 2);
-    gtk_text_view_set_left_margin(GTK_TEXT_VIEW(text_sheet), 8);
-    gtk_text_view_set_right_margin(GTK_TEXT_VIEW(text_sheet), 8);
-    gtk_text_view_set_editable(GTK_TEXT_VIEW (text_sheet), FALSE);
-    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW (text_sheet), FALSE);
+    gtk_text_view_set_pixels_above_lines (GTK_TEXT_VIEW(text_sheet), 2);
+    gtk_text_view_set_left_margin (GTK_TEXT_VIEW(text_sheet), 8);
+    gtk_text_view_set_right_margin (GTK_TEXT_VIEW(text_sheet), 8);
+    gtk_text_view_set_editable (GTK_TEXT_VIEW (text_sheet), FALSE);
+    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (text_sheet), FALSE);
     gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (text_sheet), GTK_WRAP_WORD);
     gtk_widget_show (text_sheet);
     gtk_container_add (GTK_CONTAINER (viewport), text_sheet);
 
-    sprintf(buffer, "%s\n\n", _("We would like to thank the following people for suggestions, bug reports and patches:"));
+    g_snprintf (buffer, BUFFER_SIZE, "%s\n\n", 
+                _("We would like to thank the following people for suggestions, bug reports and patches:"));
     gtk_text_buffer_insert (entry_buffer, &iter, buffer, -1);
 
     gtk_text_buffer_insert (entry_buffer, &iter, text_contributors, -1);
 
-    sprintf(buffer, "\n%s\n\n", _("Translators:"));
-    gtk_text_buffer_insert (entry_buffer, &iter, buffer, -1);
-
-    gtk_text_buffer_insert (entry_buffer, &iter, text_translators, -1);
-
     label = gtk_label_new (_("Contributors"));
     gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox2, label);
 
@@ -286,28 +353,31 @@ gchar           buffer[BUFFER_SIZE];
                   "justification", GTK_JUSTIFY_CENTER, NULL);
     gtk_text_buffer_get_iter_at_offset (entry_buffer, &iter, 0);
     text_sheet = gtk_text_view_new_with_buffer (entry_buffer);
-    gtk_text_view_set_pixels_above_lines(GTK_TEXT_VIEW(text_sheet), 2);
-    gtk_text_view_set_left_margin(GTK_TEXT_VIEW(text_sheet), 8);
-    gtk_text_view_set_right_margin(GTK_TEXT_VIEW(text_sheet), 8);
-    gtk_text_view_set_editable(GTK_TEXT_VIEW (text_sheet), FALSE);
-    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW (text_sheet), FALSE);
+    gtk_text_view_set_pixels_above_lines (GTK_TEXT_VIEW(text_sheet), 2);
+    gtk_text_view_set_left_margin (GTK_TEXT_VIEW(text_sheet), 8);
+    gtk_text_view_set_right_margin (GTK_TEXT_VIEW(text_sheet), 8);
+    gtk_text_view_set_editable (GTK_TEXT_VIEW (text_sheet), FALSE);
+    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (text_sheet), FALSE);
     gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (text_sheet), GTK_WRAP_WORD);
     gtk_widget_show (text_sheet);
     gtk_container_add (GTK_CONTAINER (viewport), text_sheet);
 
-    sprintf(buffer, "%s\n\n", _("If you like our program, send a postcard with a photo of the city where you live to:"));
+    g_snprintf (buffer, BUFFER_SIZE, "%s:\n\n", 
+                _("If you like our program, send a postcard with a photo of the city where you live to"));
     gtk_text_buffer_insert (entry_buffer, &iter, buffer, -1);
 
     gtk_text_buffer_insert (entry_buffer, &iter, text_postcards_address, -1);
 
-    sprintf(buffer, "\n\n%s\n\n", _("We have received several postcards from our happy users:"));
+    g_snprintf (buffer, BUFFER_SIZE, "\n\n%s:\n\n", 
+                _("We have received several postcards from our happy users"));
     gtk_text_buffer_insert (entry_buffer, &iter, buffer, -1);
 
     gtk_text_buffer_insert (entry_buffer, &iter, text_received_postcards, -1);
 
-    sprintf(buffer, "\n%s\n", _("Your postcards are a kind of motivation that encourages us to keep improving Kanatest :)"));
+    g_snprintf (buffer, BUFFER_SIZE, "\n%s\n", 
+                _("Your postcards are a kind of motivation that encourages us to keep improving Kanatest :)"));
     gtk_text_buffer_insert (entry_buffer, &iter, buffer, -1);
-    sprintf(buffer, "\n%s\n", _("Thank you!"));
+    g_snprintf (buffer, BUFFER_SIZE, "\n%s\n", _("Thank you!"));
     gtk_text_buffer_insert_with_tags_by_name (entry_buffer, &iter, buffer, -1, "center", NULL);
 
     label = gtk_label_new (_("Postcards"));
@@ -336,11 +406,11 @@ gchar           buffer[BUFFER_SIZE];
     gtk_text_buffer_create_tag (entry_buffer, "big",
                   "size", 12 * PANGO_SCALE, NULL);
     text_sheet = gtk_text_view_new_with_buffer (entry_buffer);
-    gtk_text_view_set_pixels_above_lines(GTK_TEXT_VIEW(text_sheet), 2);
-    gtk_text_view_set_left_margin(GTK_TEXT_VIEW(text_sheet), 8);
-    gtk_text_view_set_right_margin(GTK_TEXT_VIEW(text_sheet), 8);
-    gtk_text_view_set_editable(GTK_TEXT_VIEW (text_sheet), FALSE);
-    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW (text_sheet), FALSE);
+    gtk_text_view_set_pixels_above_lines (GTK_TEXT_VIEW(text_sheet), 2);
+    gtk_text_view_set_left_margin (GTK_TEXT_VIEW(text_sheet), 8);
+    gtk_text_view_set_right_margin (GTK_TEXT_VIEW(text_sheet), 8);
+    gtk_text_view_set_editable (GTK_TEXT_VIEW (text_sheet), FALSE);
+    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (text_sheet), FALSE);
     gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (text_sheet), GTK_WRAP_WORD);
     gtk_widget_show (text_sheet);
     gtk_container_add (GTK_CONTAINER (viewport), text_sheet);
@@ -358,13 +428,13 @@ gchar           buffer[BUFFER_SIZE];
 
     hbuttonbox = gtk_hbutton_box_new ();
     gtk_widget_show (hbuttonbox);
-    gtk_button_box_set_layout(GTK_BUTTON_BOX(hbuttonbox), GTK_BUTTONBOX_END);
+    gtk_button_box_set_layout (GTK_BUTTON_BOX(hbuttonbox), GTK_BUTTONBOX_END);
     gtk_box_pack_start (GTK_BOX (vbox1), hbuttonbox, FALSE, TRUE, 0);
 
     close_button = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
     gtk_widget_show (close_button);
     g_signal_connect (G_OBJECT (close_button), "clicked",
-                        G_CALLBACK (about_window_close_cb), NULL);
+                        G_CALLBACK (about_close_button_cb), appGUI);
     gtk_container_add (GTK_CONTAINER (hbuttonbox), close_button);
     GTK_WIDGET_SET_FLAGS (close_button, GTK_CAN_DEFAULT);
 
