@@ -23,6 +23,6 @@
 #ifndef _ABOUT_H
 #define _ABOUT_H
 
-void about_create_window (void);
+void about_create_window (GUI *appGUI);
 
 #endif /* _ABOUT_H */
