@@ -22,24 +22,23 @@
 
 #include <config.h>
 
-#include <stdio.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
 
 #include "i18n.h"
 #include "gui.h"
-#include "prefs.h"
 #include "main.h"
-#include "chart.h"
 #include "test.h"
+#include "prefs.h"
 
-static GtkWidget *chart_window = NULL;
 
-extern GtkWidget *main_window;
+/*--------------------------------------------------------------------*/
 
-gchar kana_signs_mask[CHART_ROWS][CHART_COLUMNS] = {
+gint
+get_kana_sign_mask (gint row, gint column) {
+
+gint kana_signs_mask[CHART_ROWS][CHART_COLUMNS] = {
 
     { 1, 1, 1, 1, 1 }, { 1, 1, 1, 1, 1 }, { 1, 1, 1, 1, 1 }, { 1, 1, 1, 1, 1 },
 
@@ -55,31 +54,30 @@ gchar kana_signs_mask[CHART_ROWS][CHART_COLUMNS] = {
     { 1, 0, 1, 0, 1 }, { 1, 0, 1, 0, 1 }, { 1, 0, 1, 0, 1 }, { 1, 0, 1, 0, 1 }
 };
 
-GtkWidget *labels[CHART_ROWS*CHART_COLUMNS];
+    return kana_signs_mask[row][column];
+}
 
 /*--------------------------------------------------------------------*/
 
 void
-chart_display_kanas (gint type) {
+chart_display_kanas (gint type, GUI *appGUI) {
 
 gint i, j;
 gint pos, cpos;
-gchar tmpbuf[BUFFER_SIZE], a_tmp[BUFFER_SIZE];
+gchar tmpbuf[BUFFER_SIZE];
 
     pos = cpos = 0;
 
     for(j=0; j < CHART_ROWS; j++) {
         for(i=0; i < CHART_COLUMNS; i++) {
 
-            if(kana_signs_mask[j][i]) {
+            if(get_kana_sign_mask(j, i)) {
 
-                sprintf(a_tmp, "%s", gettext(kana_signs[cpos*3+ROMAJI]));
+                g_snprintf (tmpbuf, BUFFER_SIZE, "<span font_desc='20' face='%s'>%s</span> (%s)",
+                            config.kana_font_face,
+                            get_kana_sign(cpos, type), get_kana_sign(cpos, ROMAJI));
 
-                sprintf(tmpbuf, "<span font_desc='20' face='%s'>%s</span> (%s)",
-                        config.kana_font_face,
-                        kana_signs[cpos*3+type], a_tmp);
-
-                gtk_label_set_markup (GTK_LABEL (labels[pos]), tmpbuf);
+                gtk_label_set_markup (GTK_LABEL (appGUI->chr->labels[pos]), tmpbuf);
                 cpos++;
             }
 
@@ -91,30 +89,51 @@ gchar tmpbuf[BUFFER_SIZE], a_tmp[BUFFER_SIZE];
 /*--------------------------------------------------------------------*/
 
 void
-chart_displaymode_selected_cb (GtkWidget *entry, GtkUpdateType type) {
+chart_displaymode_hiragana_selected_cb (GtkWidget *entry, gpointer user_data) {
 
-    chart_display_kanas(type);
+    GUI *appGUI = (GUI *)user_data;
+    chart_display_kanas(HIRAGANA, appGUI);
 }
 
 /*--------------------------------------------------------------------*/
 
 void
-chart_window_close_cb (void) {
+chart_displaymode_katakana_selected_cb (GtkWidget *entry, gpointer user_data) {
 
-    gdk_window_get_root_origin (chart_window->window,
+    GUI *appGUI = (GUI *)user_data;
+    chart_display_kanas(KATAKANA, appGUI);
+}
+
+/*--------------------------------------------------------------------*/
+
+void
+chart_window_close_cb (GtkWidget *widget, gpointer user_data) {
+
+    GUI *appGUI = (GUI *)user_data;
+
+    gdk_window_get_root_origin ((appGUI->chr->chart_window)->window,
                                 &config.chart_window_x, &config.chart_window_y);
-    gtk_widget_destroy (chart_window);
+    gtk_widget_destroy (appGUI->chr->chart_window);
 
-    chart_window = NULL;
+    appGUI->chr->chart_window = NULL;
+}
+
+/*--------------------------------------------------------------------*/
+
+void
+chart_delete_event_cb (GtkWidget *widget, GdkEvent *event, gpointer user_data) {
+    chart_window_close_cb (widget, user_data);
 }
 
 /*--------------------------------------------------------------------*/
 
 gint
-chart_key_press_cb (GtkWidget *widget, GdkEventKey *event, gpointer data) {
+chart_key_press_cb (GtkWidget *widget, GdkEventKey *event, gpointer user_data) {
+
+    GUI *appGUI = (GUI *)user_data;
 
     if (event->keyval == GDK_Escape) {
-        chart_window_close_cb ();
+        chart_window_close_cb (widget, appGUI);
         return TRUE;
     }
 
@@ -124,7 +143,14 @@ chart_key_press_cb (GtkWidget *widget, GdkEventKey *event, gpointer data) {
 /*--------------------------------------------------------------------*/
 
 void
-chart_create_window (void) {
+chart_close_button_cb (GtkWidget *widget, gpointer user_data) {
+    chart_window_close_cb (widget, user_data);
+}
+
+/*--------------------------------------------------------------------*/
+
+void
+chart_create_window (GUI *appGUI) {
 
 GtkWidget   *vbox1;
 GtkWidget   *vbox2;
@@ -142,26 +168,26 @@ GSList      *radiobutton_group = NULL;
 gint        i, j, pos;
 
 
-    if (chart_window) return;
+    if (appGUI->chr->chart_window) return;
 
-    chart_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-    gtk_window_set_transient_for(GTK_WINDOW(chart_window),GTK_WINDOW(main_window));
-    gtk_window_set_resizable (GTK_WINDOW (chart_window), TRUE);
-    gtk_container_set_border_width (GTK_CONTAINER (chart_window), 12);
-    gtk_window_set_title (GTK_WINDOW (chart_window), _("Kana chart"));
-    gtk_widget_set_size_request (chart_window, -1, 600);
+    appGUI->chr->chart_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+    gtk_window_set_transient_for (GTK_WINDOW(appGUI->chr->chart_window),GTK_WINDOW(appGUI->main_window));
+    gtk_window_set_resizable (GTK_WINDOW (appGUI->chr->chart_window), TRUE);
+    gtk_container_set_border_width (GTK_CONTAINER (appGUI->chr->chart_window), 12);
+    gtk_window_set_title (GTK_WINDOW (appGUI->chr->chart_window), _("Kana chart"));
+    gtk_widget_set_size_request (appGUI->chr->chart_window, -1, 600);
 
-    g_signal_connect (G_OBJECT (chart_window), "delete_event",
-                        G_CALLBACK(chart_window_close_cb), NULL);
-    g_signal_connect (G_OBJECT(chart_window), "key_press_event",
-                        G_CALLBACK(chart_key_press_cb), NULL);
+    g_signal_connect (G_OBJECT (appGUI->chr->chart_window), "delete_event",
+                        G_CALLBACK(chart_delete_event_cb), appGUI);
+    g_signal_connect (G_OBJECT(appGUI->chr->chart_window), "key_press_event",
+                        G_CALLBACK(chart_key_press_cb), appGUI);
 
-    gtk_window_move (GTK_WINDOW (chart_window),
+    gtk_window_move (GTK_WINDOW (appGUI->chr->chart_window),
                                 config.chart_window_x, config.chart_window_y);
 
     vbox2 = gtk_vbox_new (FALSE, 0);
     gtk_widget_show (vbox2);
-    gtk_container_add (GTK_CONTAINER (chart_window), vbox2);
+    gtk_container_add (GTK_CONTAINER (appGUI->chr->chart_window), vbox2);
 
     scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
     gtk_widget_show (scrolledwindow);
@@ -185,7 +211,7 @@ gint        i, j, pos;
     for(j=0; j < CHART_ROWS; j++) {
         for(i=0; i < CHART_COLUMNS; i++) {
 
-            if(kana_signs_mask[j][i]) {
+            if(get_kana_sign_mask(j, i)) {
 
                 frames[pos] = gtk_frame_new (NULL);
                 gtk_widget_show (frames[pos]);
@@ -195,10 +221,10 @@ gint        i, j, pos;
                 gtk_container_set_border_width (GTK_CONTAINER (frames[pos]), 2);
                 gtk_frame_set_shadow_type (GTK_FRAME (frames[pos]), GTK_SHADOW_ETCHED_OUT);
 
-                labels[pos] = gtk_label_new (NULL);
-                gtk_widget_show (labels[pos]);
-                gtk_container_add (GTK_CONTAINER (frames[pos]), labels[pos]);
-                gtk_misc_set_padding (GTK_MISC (labels[pos]), 0, 4);
+                appGUI->chr->labels[pos] = gtk_label_new (NULL);
+                gtk_widget_show (appGUI->chr->labels[pos]);
+                gtk_container_add (GTK_CONTAINER (frames[pos]), appGUI->chr->labels[pos]);
+                gtk_misc_set_padding (GTK_MISC (appGUI->chr->labels[pos]), 0, 4);
             }
 
             pos++;
@@ -212,8 +238,7 @@ gint        i, j, pos;
     hiragana_radiobutton = gtk_radio_button_new_with_mnemonic (NULL, _("Hiragana"));
     gtk_widget_show (hiragana_radiobutton);
     g_signal_connect (G_OBJECT (hiragana_radiobutton), "clicked",
-                      G_CALLBACK (chart_displaymode_selected_cb),
-                      GINT_TO_POINTER (HIRAGANA));
+                      G_CALLBACK (chart_displaymode_hiragana_selected_cb), appGUI);
     gtk_box_pack_start (GTK_BOX (hbox1), hiragana_radiobutton, FALSE, FALSE, 0);
     gtk_container_set_border_width (GTK_CONTAINER (hiragana_radiobutton), 4);
     gtk_radio_button_set_group (GTK_RADIO_BUTTON (hiragana_radiobutton), radiobutton_group);
@@ -223,8 +248,7 @@ gint        i, j, pos;
     katakana_radiobutton = gtk_radio_button_new_with_mnemonic (NULL, _("Katakana"));
     gtk_widget_show (katakana_radiobutton);
     g_signal_connect (G_OBJECT (katakana_radiobutton), "clicked",
-                      G_CALLBACK (chart_displaymode_selected_cb),
-                      GINT_TO_POINTER (KATAKANA));
+                      G_CALLBACK (chart_displaymode_katakana_selected_cb), appGUI);
     gtk_box_pack_start (GTK_BOX (hbox1), katakana_radiobutton, FALSE, FALSE, 0);
     gtk_container_set_border_width (GTK_CONTAINER (katakana_radiobutton), 4);
     gtk_radio_button_set_group (GTK_RADIO_BUTTON (katakana_radiobutton), radiobutton_group);
@@ -232,7 +256,7 @@ gint        i, j, pos;
 
     hbuttonbox = gtk_hbutton_box_new ();
     gtk_widget_show (hbuttonbox);
-    gtk_button_box_set_layout(GTK_BUTTON_BOX(hbuttonbox), GTK_BUTTONBOX_END);
+    gtk_button_box_set_layout (GTK_BUTTON_BOX(hbuttonbox), GTK_BUTTONBOX_END);
     gtk_box_pack_end (GTK_BOX (vbox2), hbuttonbox, FALSE, TRUE, 0);
 
     hseparator = gtk_hseparator_new ();
@@ -242,13 +266,13 @@ gint        i, j, pos;
     close_button = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
     gtk_widget_show (close_button);
     g_signal_connect (G_OBJECT (close_button), "clicked",
-                        G_CALLBACK (chart_window_close_cb), NULL);
+                        G_CALLBACK (chart_close_button_cb), appGUI);
     gtk_container_add (GTK_CONTAINER (hbuttonbox), close_button);
     GTK_WIDGET_SET_FLAGS (close_button, GTK_CAN_DEFAULT);
 
-    chart_display_kanas(HIRAGANA);
+    chart_display_kanas(HIRAGANA, appGUI);
 
-    gtk_widget_show (chart_window);
+    gtk_widget_show (appGUI->chr->chart_window);
     gtk_widget_grab_default (close_button);
 
 }
