@@ -40,6 +40,6 @@ enum {
 };
 
 void update_example_text(void);
-void options_create_window (void);
+void options_create_window (GUI *appGUI);
 
 #endif /* _OPTIONS_H */
