@@ -25,9 +25,11 @@
 #define NUMBER_OF_COLUMNS   8
 
 #define STATS_NAME          "kanatest_stats"
-#define STATS_FILENAME      "kanatest-stats.xml"
+#define STATS_FILENAME      "statistics.xml"
+#define STATS_FILENAME_OLD  "kanatest-stats.xml"
+#define KANAS_FILENAME      "statistics-kanas.xml"
 
-struct stats_entry {
+typedef struct {
     guint date_day;
     guint date_month;
     guint date_year;
@@ -39,16 +41,16 @@ struct stats_entry {
     guint test_questions;
     guint test_correct_answers;
     guint test_repeat_mode;
-};
+} stats_entry;
 
-void    stats_create_window (void);
-void    stats_info_no_stats (void);
-void    stats_read_list     (void);
-void    stats_write_list    (void);
-void    stats_add_entry     (guint day, guint month, guint year, guint hour, guint minute, guint time, 
-                             guint mode, guint kana_set, guint questions, guint correct_answers, guint repeat_mode);
-gint    stats_get_records   (void);
-void    stats_remove_list   (void);
+void    stats_create_window (GUI *appGUI);
+
+gint    stats_get_records   (GUI *appGUI);
+void    stats_remove_list   (GUI *appGUI);
+void    stats_free_list     (GUI *appGUI);
+void    stats_add_entry     (stats_entry *entry, GUI *appGUI);
+void    stats_read_list     (gchar *stats_filename, gchar *config_dirname, GUI *appGUI);
+void    stats_write_list    (gchar *stats_filename, gchar *config_dirname, GUI *appGUI);
 
 #endif /* _STATS_H */
 
