@@ -20,39 +20,39 @@
  */
 
 
-#include <stdio.h>
 #include <stdlib.h>
-#include <time.h>
-#include <unistd.h>
 #include <string.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/time.h>
 
-#include <glib.h>
 #include <gtk/gtk.h>
 #include <config.h>
 
 #include "gui.h"
-#include "prefs.h"
 #include "main.h"
 #include "test.h"
+#include "prefs.h"
 #include "stats.h"
 #include "options.h"
 #include "i18n.h"
 
-gint question_counter, right_answer_counter, wrong_answer_counter;
-gint max_entries_in_test;
 
-guint test_time;
+/*--------------------------------------------------------------------*/
 
-gint questions_table[MAX_NUMBER_OF_SIGNS];
+gchar *
+get_test_mode_name (gint mode) {
 
-extern gboolean any_key;
-extern gboolean test_state;
-extern GtkWidget *main_window;
+gchar *test_mode_name[] = {
+    NULL, N_("Hiragana"), N_("Katakana"), N_("Mixed")
+};
 
-gchar *kana_signs_names[] = {
+    return gettext(test_mode_name[mode]);
+}
+
+/*--------------------------------------------------------------------*/
+
+gchar *
+get_kana_set_name (gint set) {
+
+gchar *kana_set_name[] = {
     N_("All kanas"),
     N_("Basic kanas"),
     N_("A-I-U-E-O"),
@@ -70,104 +70,13 @@ gchar *kana_signs_names[] = {
     N_("User-defined lesson")
 };
 
-gchar *kana_mode_names[] = {
-    NULL, N_("Hiragana"), N_("Katakana"), N_("Mixed")
-};
+    return gettext(kana_set_name[set]);
+}
 
-gint kana_signs_set_0_idx[NUMBER_OF_SIGNS]; /* All kanas */
+/*--------------------------------------------------------------------*/
 
-gint kana_signs_set_1_idx[] = {
-    0, 1, 2, 3, 4,                  /* A-I-U-E-O */
-    5, 6, 7, 8, 9,                  /* KA-KI-KU-KE-KO */
-    10, 11, 12, 13, 14,             /* SA-SHI-SU-SE-SO */
-    15, 16, 17, 18, 19,             /* TA-CHI-TSU-TE-TO */
-    20, 21, 22, 23, 24,             /* NA-NI-NU-NE-NO */
-    25, 26, 27, 28, 29,             /* HA-HI-FU-HE-HO */
-    30, 31, 32, 33, 34,             /* MA-MI-MU-ME-MO */
-    35, 36, 37,                     /* YA-YU-YO */
-    38, 39, 40, 41, 42,             /* RA-RI-RU-RE-RO */
-    43, 44, 45                      /* WA-WO-N */
-};
-
-gint kana_signs_set_2_idx[] = {     /* A-I-U-E-O */
-    0, 1, 2, 3, 4
-};
-
-gint kana_signs_set_3_idx[] = {     /* KA-KI-KU-KE-KO */
-    5, 6, 7, 8, 9
-};
-
-gint kana_signs_set_4_idx[] = {     /* SA-SHI-SU-SE-SO */
-    10, 11, 12, 13, 14
-};
-
-gint kana_signs_set_5_idx[] = {     /* TA-CHI-TSU-TE-TO */
-    15, 16, 17, 18, 19
-};
-
-gint kana_signs_set_6_idx[] = {     /* NA-NI-NU-NE-NO */
-    20, 21, 22, 23, 24
-};
-
-gint kana_signs_set_7_idx[] = {     /* HA-HI-FU-HE-HO */
-    25, 26, 27, 28, 29
-};
-
-gint kana_signs_set_8_idx[] = {     /* MA-MI-MU-ME-MO */
-    30, 31, 32, 33, 34
-};
-
-gint kana_signs_set_9_idx[] = {     /* YA-YU-YO */
-    35, 36, 37
-};
-
-gint kana_signs_set_10_idx[] = {    /* RA-RI-RU-RE-RO */
-    38, 39, 40, 41, 42
-};
-
-gint kana_signs_set_11_idx[] = {    /* WA-WO-N */
-    43, 44, 45
-};
-
-gint kana_signs_set_12_idx[] = {    /* Combo letters (part 1) */
-    46, 47, 48, 49, 50, 51, 52, 53, 54, 55,
-    56, 57, 58, 59, 60, 61, 62, 63, 64, 65,
-    66, 67, 68, 69, 70
-};
-
-gint kana_signs_set_13_idx[] = {    /* Combo letters (part 2) */
-    71, 72, 73, 74, 75, 76, 77, 78, 79, 80,
-    81, 82, 83, 84, 85, 86, 87, 88, 89, 90,
-    91, 92, 93, 94, 95, 96, 97, 98, 99, 100,
-    101, 102, 103
-};
-
-gint kana_signs_set_14_idx[NUMBER_OF_SIGNS]; /* user-defined-lessons - all kanas reserved */
-
-gint *kana_signs_set_tabs[] = {
-    kana_signs_set_0_idx,
-    kana_signs_set_1_idx,
-    kana_signs_set_2_idx,
-    kana_signs_set_3_idx,
-    kana_signs_set_4_idx,
-    kana_signs_set_5_idx,
-    kana_signs_set_6_idx,
-    kana_signs_set_7_idx,
-    kana_signs_set_8_idx,
-    kana_signs_set_9_idx,
-    kana_signs_set_10_idx,
-    kana_signs_set_11_idx,
-    kana_signs_set_12_idx,
-    kana_signs_set_13_idx,
-    kana_signs_set_14_idx
-};
-
-/* -1 is for number of selected kanas  - offset 14 */
-#define USER_DEFINED_LEN_OFFSET 14
-
-gint kana_signs_set_len[] = {
-    104, 46, 5, 5, 5, 5, 5, 5, 5, 3, 5, 3, 25, 33, -1
-};
+gchar *
+get_kana_sign (gint index, gint kana_type) {
 
 /* romanji | hiragana | katakana */
 
@@ -237,49 +146,157 @@ gchar *kana_signs[] = {
 
 };
 
+    return gettext(kana_signs[3 * index + kana_type]);
+}
+
 /*--------------------------------------------------------------------*/
 
 void
-test_init (void) {
+test_init (GUI *appGUI) {
 
     gint i, pos;
 
     pos = 0;
 
     for (i=0; i < NUMBER_OF_SIGNS; i++) {
-        kana_signs_set_0_idx[i] = i;
+        appGUI->tst->kana_signs_set_0_idx[i] = i;
 
         if (config.user_defined_lesson[i] == '+') {
-            kana_signs_set_14_idx[pos++] = i;
+            appGUI->tst->kana_signs_set_14_idx[pos++] = i;
         }
     }
 
-    kana_signs_set_len[USER_DEFINED_LEN_OFFSET] = pos;   /* replace -1 with number of selected kanas */
+    appGUI->tst->kana_set_len[0] = KANA_SET_0_LEN;
+    appGUI->tst->kana_set_len[1] = KANA_SET_1_LEN;
+    appGUI->tst->kana_set_len[2] = KANA_SET_2_LEN;
+    appGUI->tst->kana_set_len[3] = KANA_SET_3_LEN;
+    appGUI->tst->kana_set_len[4] = KANA_SET_4_LEN;
+    appGUI->tst->kana_set_len[5] = KANA_SET_5_LEN;
+    appGUI->tst->kana_set_len[6] = KANA_SET_6_LEN;
+    appGUI->tst->kana_set_len[7] = KANA_SET_7_LEN;
+    appGUI->tst->kana_set_len[8] = KANA_SET_8_LEN;
+    appGUI->tst->kana_set_len[9] = KANA_SET_9_LEN;
+    appGUI->tst->kana_set_len[10] = KANA_SET_10_LEN;
+    appGUI->tst->kana_set_len[11] = KANA_SET_11_LEN;
+    appGUI->tst->kana_set_len[12] = KANA_SET_12_LEN;
+    appGUI->tst->kana_set_len[13] = KANA_SET_13_LEN;
 
-    max_entries_in_test = kana_signs_set_len[config.kana_set] * ((config.kana_mode == MIXED) ? 2 : 1);
+    appGUI->tst->kana_set_len[USER_DEFINED_LEN_OFFSET] = pos;   /* replace -1 with number of selected kanas */
 
-    question_counter = 0;
-    wrong_answer_counter = 0;
-    right_answer_counter = 0;
+    appGUI->tst->max_entries_in_test = appGUI->tst->kana_set_len[config.kana_set] * ((config.kana_mode == MIXED) ? 2 : 1);
+
+    appGUI->tst->question_counter = 0;
+    appGUI->tst->wrong_answer_counter = 0;
+    appGUI->tst->right_answer_counter = 0;
 }
 
 /*--------------------------------------------------------------------*/
 
 void
-test_generate_tables (gint number_of_questions) {
+test_generate_tables (gint number_of_questions, GUI *appGUI) {
 
-    gint i, tmp, pos;
-    gint *table;
+gint kana_signs_set_1_idx[] = {
+    0, 1, 2, 3, 4,                  /* A-I-U-E-O */
+    5, 6, 7, 8, 9,                  /* KA-KI-KU-KE-KO */
+    10, 11, 12, 13, 14,             /* SA-SHI-SU-SE-SO */
+    15, 16, 17, 18, 19,             /* TA-CHI-TSU-TE-TO */
+    20, 21, 22, 23, 24,             /* NA-NI-NU-NE-NO */
+    25, 26, 27, 28, 29,             /* HA-HI-FU-HE-HO */
+    30, 31, 32, 33, 34,             /* MA-MI-MU-ME-MO */
+    35, 36, 37,                     /* YA-YU-YO */
+    38, 39, 40, 41, 42,             /* RA-RI-RU-RE-RO */
+    43, 44, 45                      /* WA-WO-N */
+};
 
-    table = kana_signs_set_tabs[config.kana_set];
+gint kana_signs_set_2_idx[] = {     /* A-I-U-E-O */
+    0, 1, 2, 3, 4
+};
+
+gint kana_signs_set_3_idx[] = {     /* KA-KI-KU-KE-KO */
+    5, 6, 7, 8, 9
+};
+
+gint kana_signs_set_4_idx[] = {     /* SA-SHI-SU-SE-SO */
+    10, 11, 12, 13, 14
+};
+
+gint kana_signs_set_5_idx[] = {     /* TA-CHI-TSU-TE-TO */
+    15, 16, 17, 18, 19
+};
+
+gint kana_signs_set_6_idx[] = {     /* NA-NI-NU-NE-NO */
+    20, 21, 22, 23, 24
+};
+
+gint kana_signs_set_7_idx[] = {     /* HA-HI-FU-HE-HO */
+    25, 26, 27, 28, 29
+};
+
+gint kana_signs_set_8_idx[] = {     /* MA-MI-MU-ME-MO */
+    30, 31, 32, 33, 34
+};
+
+gint kana_signs_set_9_idx[] = {     /* YA-YU-YO */
+    35, 36, 37
+};
+
+gint kana_signs_set_10_idx[] = {    /* RA-RI-RU-RE-RO */
+    38, 39, 40, 41, 42
+};
+
+gint kana_signs_set_11_idx[] = {    /* WA-WO-N */
+    43, 44, 45
+};
+
+gint kana_signs_set_12_idx[] = {    /* Combo letters (part 1) */
+    46, 47, 48, 49, 50, 51, 52, 53, 54, 55,
+    56, 57, 58, 59, 60, 61, 62, 63, 64, 65,
+    66, 67, 68, 69, 70
+};
+
+gint kana_signs_set_13_idx[] = {    /* Combo letters (part 2) */
+    71, 72, 73, 74, 75, 76, 77, 78, 79, 80,
+    81, 82, 83, 84, 85, 86, 87, 88, 89, 90,
+    91, 92, 93, 94, 95, 96, 97, 98, 99, 100,
+    101, 102, 103
+};
+
+gint *kana_signs_set_tabs[] = {
+    NULL,                   /* kana_signs_set_0_idx */
+    kana_signs_set_1_idx,
+    kana_signs_set_2_idx,
+    kana_signs_set_3_idx,
+    kana_signs_set_4_idx,
+    kana_signs_set_5_idx,
+    kana_signs_set_6_idx,
+    kana_signs_set_7_idx,
+    kana_signs_set_8_idx,
+    kana_signs_set_9_idx,
+    kana_signs_set_10_idx,
+    kana_signs_set_11_idx,
+    kana_signs_set_12_idx,
+    kana_signs_set_13_idx,
+    NULL                    /* kana_signs_set_14_idx */
+};
+
+gint i, tmp, pos;
+gint *table;
+
+    if (config.kana_set == 0) {
+        table = appGUI->tst->kana_signs_set_0_idx;
+    } else if (config.kana_set == USER_DEFINED_LEN_OFFSET) {
+        table = appGUI->tst->kana_signs_set_14_idx;
+    } else {
+        table = kana_signs_set_tabs[config.kana_set];
+    }
 
     /* fill questions and answers tables */
     for (i = 0; i < number_of_questions; i++) {
 
-        if (i >= kana_signs_set_len[config.kana_set]) {
-            questions_table[i] = table[i-kana_signs_set_len[config.kana_set]] + MIXED_SEPARATOR;
+        if (i >= appGUI->tst->kana_set_len[config.kana_set]) {
+            appGUI->tst->questions_table[i] = table[i-appGUI->tst->kana_set_len[config.kana_set]] + MIXED_SEPARATOR;
         } else {
-            questions_table[i] = table[i];
+            appGUI->tst->questions_table[i] = table[i];
         }
 
     }
@@ -287,9 +304,9 @@ test_generate_tables (gint number_of_questions) {
     /* do a decent random permutation */
     for(i = number_of_questions-1; i > 0; --i) {
         pos = lrand48() % (i+1);
-        tmp = questions_table[i];
-        questions_table[i] = questions_table[pos];
-        questions_table[pos] = tmp;
+        tmp = appGUI->tst->questions_table[i];
+        appGUI->tst->questions_table[i] = appGUI->tst->questions_table[pos];
+        appGUI->tst->questions_table[pos] = tmp;
     }
 }
 
@@ -298,20 +315,21 @@ test_generate_tables (gint number_of_questions) {
 gchar*
 test_sec2str (guint seconds, gboolean mshort) {
 
-    static gchar obuffer[BUFFER_SIZE];
-    guint minutes;
+static gchar obuffer[BUFFER_SIZE];
+guint minutes;
 
     minutes = seconds / 60;
     seconds = seconds % 60;
 
     if (mshort == TRUE) {
-        sprintf (obuffer, "%02d:%02d", minutes, seconds);
+        g_snprintf (obuffer, BUFFER_SIZE, "%02d:%02d", minutes, seconds);
     } else {
         if (minutes > 0) {
-            sprintf (obuffer, "%d %s %d %s", minutes, ngettext("minute and", "minutes and", minutes),
-                                             seconds, ngettext("second", "seconds", seconds));
+            g_snprintf (obuffer, BUFFER_SIZE, "%d %s %d %s", 
+                        minutes, ngettext ("minute and", "minutes and", minutes),
+                        seconds, ngettext ("second", "seconds", seconds));
         } else {
-            sprintf (obuffer, "%d %s", seconds, ngettext("second", "seconds", seconds));
+            g_snprintf (obuffer, BUFFER_SIZE, "%d %s", seconds, ngettext ("second", "seconds", seconds));
         }
     }
 
@@ -321,16 +339,16 @@ test_sec2str (guint seconds, gboolean mshort) {
 /*--------------------------------------------------------------------*/
 
 void
-test_check_answer (gchar *user_answer) {
+test_check_answer (gchar *user_answer, GUI *appGUI) {
 
-    gchar tmp_a[BUFFER_SIZE], tmp_b[BUFFER_SIZE];
-    gint kana_number, old_counter;
-    gboolean correct_answer, end;
-    gchar **r_answers;
-    gint n_answ;
+gchar tmp_a[BUFFER_SIZE], tmp_b[BUFFER_SIZE];
+gint kana_number, old_counter;
+gboolean correct_answer, end;
+gchar **r_answers;
+gint n_answ;
 
-    kana_number = questions_table[question_counter] % MIXED_SEPARATOR;
-    r_answers = g_strsplit (gettext(kana_signs[kana_number*3]), ANSWER_DELIMITER, MAX_RANSWERS);
+    kana_number = appGUI->tst->questions_table[appGUI->tst->question_counter] % MIXED_SEPARATOR;
+    r_answers = g_strsplit (get_kana_sign (kana_number, ROMAJI), ANSWER_DELIMITER, MAX_RANSWERS);
 	n_answ=0;
 	correct_answer = FALSE;
 
@@ -344,19 +362,31 @@ test_check_answer (gchar *user_answer) {
 
     g_strfreev (r_answers);
 
+    if (config.kana_mode == HIRAGANA) {
+        appGUI->sts->hiragana_counters[kana_number]++;
+    } else if (config.kana_mode == KATAKANA) {
+        appGUI->sts->katakana_counters[kana_number]++;
+    } else if (config.kana_mode == MIXED) {
+        if (appGUI->tst->questions_table[appGUI->tst->question_counter] >= MIXED_SEPARATOR) {
+            appGUI->sts->katakana_counters[kana_number]++;
+        } else {
+            appGUI->sts->hiragana_counters[kana_number]++;
+        }
+    }
+
     if (correct_answer == FALSE) {
 
         /* WRONG ANSWER */
 
         if (config.ca_timeout != TO_DISABLED) {
 
-            any_key = FALSE;
+            appGUI->tst->any_key = FALSE;
 
-            gtk_widget_set_sensitive (romaji_entry, FALSE);
-            gtk_widget_set_sensitive (stop_button, FALSE);
+            gtk_widget_set_sensitive (appGUI->romaji_entry, FALSE);
+            gtk_widget_set_sensitive (appGUI->stop_button, FALSE);
 
-            gui_display_kana (questions_table[question_counter], ROMAJI);
-            while (g_main_context_iteration(NULL, FALSE));
+            gui_display_kana (appGUI->tst->questions_table[appGUI->tst->question_counter], ROMAJI, appGUI);
+            while (g_main_context_iteration (NULL, FALSE));
 
             if (config.ca_timeout != TO_ANYKEY) {
 
@@ -364,62 +394,77 @@ test_check_answer (gchar *user_answer) {
 
             } else {    /* waiting for any key */
 
-                strcpy (tmp_a, gtk_entry_get_text (GTK_ENTRY(romaji_entry)));
-                sprintf(tmp_b, "%s (Press any key)", tmp_a);
-                gtk_entry_set_max_length(GTK_ENTRY(romaji_entry), 32);
+                g_strlcpy (tmp_a, gtk_entry_get_text (GTK_ENTRY(appGUI->romaji_entry)), BUFFER_SIZE);
+                g_snprintf (tmp_b, BUFFER_SIZE, "%s (Press any key)", tmp_a);
+                gtk_entry_set_max_length (GTK_ENTRY(appGUI->romaji_entry), 32);
 
-                while (any_key != TRUE) {
-                    gtk_entry_set_text (GTK_ENTRY(romaji_entry), tmp_b);
-                    while (g_main_context_iteration(NULL, FALSE));
+                while (appGUI->tst->any_key != TRUE && appGUI->tst->test_state != FALSE) {
+                    gtk_entry_set_text (GTK_ENTRY(appGUI->romaji_entry), tmp_b);
+                    while (g_main_context_iteration (NULL, FALSE));
                     g_usleep (250000);
-                    if (any_key) {
+                    if (appGUI->tst->any_key) {
                         break;
                     }
-                    gtk_entry_set_text (GTK_ENTRY(romaji_entry), tmp_a);
-                    while (g_main_context_iteration(NULL, FALSE));
+                    gtk_entry_set_text (GTK_ENTRY(appGUI->romaji_entry), tmp_a);
+                    while (g_main_context_iteration (NULL, FALSE));
                     g_usleep (200000);
                 }
 
-                gtk_entry_set_max_length(GTK_ENTRY(romaji_entry), 3);
-                gtk_entry_set_text (GTK_ENTRY(romaji_entry), tmp_a);
-                while (g_main_context_iteration(NULL, FALSE));
+                gtk_entry_set_max_length (GTK_ENTRY(appGUI->romaji_entry), 3);
+                gtk_entry_set_text (GTK_ENTRY(appGUI->romaji_entry), tmp_a);
+                while (g_main_context_iteration (NULL, FALSE));
+    
+                if (appGUI->tst->test_state == FALSE) return;
             }
 
-            gtk_widget_set_sensitive (romaji_entry, TRUE);
-            gtk_widget_set_sensitive (stop_button, TRUE);
+            gtk_widget_set_sensitive (appGUI->romaji_entry, TRUE);
+            gtk_widget_set_sensitive (appGUI->stop_button, TRUE);
 
-            gtk_widget_grab_focus (romaji_entry);
+            gtk_widget_grab_focus (appGUI->romaji_entry);
 
         }
 
-        wrong_answer_counter++;
+        appGUI->tst->wrong_answer_counter++;
 
     } else {
-        questions_table[question_counter] = -1;
-        right_answer_counter++;
+
+        if (config.kana_mode == HIRAGANA) {
+            appGUI->sts->correct_hiragana_counters[kana_number]++;
+        } else if (config.kana_mode == KATAKANA) {
+            appGUI->sts->correct_katakana_counters[kana_number]++;
+        } else if (config.kana_mode == MIXED) {
+            if (appGUI->tst->questions_table[appGUI->tst->question_counter] >= MIXED_SEPARATOR) {
+                appGUI->sts->correct_katakana_counters[kana_number]++;
+            } else {
+                appGUI->sts->correct_hiragana_counters[kana_number]++;
+            }
+        }
+
+        appGUI->tst->questions_table[appGUI->tst->question_counter] = -1;
+        appGUI->tst->right_answer_counter++;
     }
 
-    old_counter = question_counter;
+    old_counter = appGUI->tst->question_counter;
     end = FALSE;
 
     while (!end) {
 
-        question_counter = (question_counter + 1) % max_entries_in_test;    /* next question */
+        appGUI->tst->question_counter = (appGUI->tst->question_counter + 1) % appGUI->tst->max_entries_in_test;    /* next question */
 
         if (config.repeat_mode == REPEAT_WRONG) {
 
-            if (questions_table[question_counter] != -1) {
+            if (appGUI->tst->questions_table[appGUI->tst->question_counter] != -1) {
                 break;          /* this has not yet been answered correctly */
             }
 
-            if (old_counter == question_counter) {
+            if (old_counter == appGUI->tst->question_counter) {
                 end = TRUE;     /* every question is answered correctly */
             }
 
-        }  else if (0 == question_counter) { /* all questions ? */
+        }  else if (0 == appGUI->tst->question_counter) { /* all questions ? */
 
             if (config.repeat_mode == REPEAT_ALL) {
-                test_generate_tables (max_entries_in_test);
+                test_generate_tables (appGUI->tst->max_entries_in_test, appGUI);
             } else {
                 end = TRUE;
             }
@@ -430,86 +475,102 @@ test_check_answer (gchar *user_answer) {
     }
 
     if (end) {
-        test_info(); /* finish! */
+        test_info (appGUI); /* finish! */
     } else {
-        gui_set_progress();
-        gui_display_kana (questions_table[question_counter], config.kana_mode);
+        gui_set_progress (appGUI);
+        gui_display_kana (appGUI->tst->questions_table[appGUI->tst->question_counter], config.kana_mode, appGUI);
     }
 }
 
 /*--------------------------------------------------------------------*/
 
 void
-test_info(void) {
+test_info(GUI *appGUI) {
 
     GtkWidget *info_dialog;
     gchar   message[BUFFER_SIZE];
     struct tm   *timer;
     time_t      tmm;
-    guint answer_counter = right_answer_counter + wrong_answer_counter;
+    guint answer_counter = appGUI->tst->right_answer_counter + appGUI->tst->wrong_answer_counter;
+    stats_entry *entry = NULL;
 
-    if (test_state == FALSE) return;
+    if (appGUI->tst->test_state == FALSE) return;
 
-    gui_disable_test ();
+    gui_disable_test (appGUI);
 
     if (answer_counter == 0) return;
-    test_time = difftime (time(NULL), start_time);
 
     if (config.repeat_mode == REPEAT_ALL) {
 
-        snprintf(message, BUFFER_SIZE,
-                "<span font_desc='16'><b>%s</b></span>\n\n"
-                "<span font_desc='12'>%s %d (%d %s)\n"
-                "%s %d\n%s %d\n"
-                "\n%s %.f %%\n"
-                "\n%s %s\n</span>",
-                _("Test has finished!"), _("Total questions:"),
-                answer_counter,
-                max_entries_in_test,
-                _("unique"),
-                _("Correct answers:"),
-                right_answer_counter,
-                _("Wrong answers:"),
-                wrong_answer_counter,
-                _("Correctness ratio:"),
-                (gfloat)(right_answer_counter)/answer_counter*100.0,
-                _("Drilling-time:"),
-                test_sec2str(test_time, FALSE));
+        g_snprintf (message, BUFFER_SIZE,
+                    "<span font_desc='16'><b>%s</b></span>\n\n"
+                    "<span font_desc='12'>%s: %d (%d %s)\n"
+                    "%s: %d\n%s: %d\n"
+                    "\n%s: %.f %%\n"
+                    "\n%s: %s\n</span>",
+                    _("Test has finished!"), _("Total questions"),
+                    answer_counter,
+                    appGUI->tst->max_entries_in_test,
+                    _("unique"),
+                    _("Correct answers"),
+                    appGUI->tst->right_answer_counter,
+                    _("Wrong answers"),
+                    appGUI->tst->wrong_answer_counter,
+                    _("Correctness ratio"),
+                    (gfloat)(appGUI->tst->right_answer_counter) / answer_counter * 100.0,
+                    _("Drilling-time"),
+                    test_sec2str (appGUI->time_counter, FALSE));
     } else {
 
-        snprintf(message, BUFFER_SIZE,
-                "<span font_desc='16'><b>%s</b></span>\n\n"
-                "<span font_desc='12'>%s %d\t\t\t\n"
-                "%s %d\n%s %d\n"
-                "\n%s %.f %%\n"
-                "\n%s %s\n</span>",
-                _("Test has finished!"), _("Total questions:"),
-                answer_counter,
-                _("Correct answers:"),
-                right_answer_counter,
-                _("Wrong answers:"),
-                wrong_answer_counter,
-                _("Correctness ratio:"),
-                (gfloat)(right_answer_counter)/answer_counter*100.0,
-                _("Drilling-time:"),
-                test_sec2str(test_time, FALSE));
+        g_snprintf (message, BUFFER_SIZE,
+                    "<span font_desc='16'><b>%s</b></span>\n\n"
+                    "<span font_desc='12'>%s: %d\t\t\t\n"
+                    "%s: %d\n%s: %d\n"
+                    "\n%s: %.f %%\n"
+                    "\n%s: %s\n</span>",
+                    _("Test has finished!"), _("Total questions"),
+                    answer_counter,
+                    _("Correct answers"),
+                    appGUI->tst->right_answer_counter,
+                    _("Wrong answers"),
+                    appGUI->tst->wrong_answer_counter,
+                    _("Correctness ratio"),
+                    (gfloat)(appGUI->tst->right_answer_counter) / answer_counter * 100.0,
+                    _("Drilling-time"),
+                    test_sec2str (appGUI->time_counter, FALSE));
     }
 
-    info_dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW(main_window),
-                                          GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
-                                          GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE, message);
+    info_dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW(appGUI->main_window),
+                                                      GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
+                                                      GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE, message);
 
-    gtk_window_set_title(GTK_WINDOW(info_dialog), _("Information"));
+    gtk_window_set_title (GTK_WINDOW(info_dialog), _("Information"));
     gtk_widget_show (info_dialog);
-    gtk_dialog_run(GTK_DIALOG(info_dialog));
-    gtk_widget_destroy(info_dialog);
+    gtk_dialog_run (GTK_DIALOG(info_dialog));
+    gtk_widget_destroy (info_dialog);
 
-    tmm = time(NULL); 
-    timer = localtime(&tmm);
+    entry = g_new0 (stats_entry, 1);
 
-    stats_add_entry (timer->tm_mday, timer->tm_mon, timer->tm_year+1900, timer->tm_hour, timer->tm_min,
-                     (guint) test_time, config.kana_mode, config.kana_set,
-                     answer_counter,right_answer_counter, config.repeat_mode);
+    if (entry != NULL) {
+
+        tmm = time (NULL); 
+        timer = localtime (&tmm);
+
+        entry->date_day = timer->tm_mday;
+        entry->date_month = timer->tm_mon;
+        entry->date_year = timer->tm_year+1900;
+        entry->date_hour = timer->tm_hour;
+        entry->date_minute = timer->tm_min;
+        entry->test_time = appGUI->time_counter;
+        entry->test_mode = config.kana_mode;
+        entry->test_kana_set = config.kana_set;
+        entry->test_questions = answer_counter;
+        entry->test_correct_answers = appGUI->tst->right_answer_counter;
+        entry->test_repeat_mode = config.repeat_mode;
+
+        stats_add_entry (entry, appGUI);
+        stats_write_list (STATS_FILENAME, CONFIG_DIRNAME, appGUI);
+    }
 }
 
 /*--------------------------------------------------------------------*/
