@@ -22,29 +22,15 @@
 #ifndef _TEST_H
 #define _TEST_H
 
-#define			NUMBER_OF_SIGNS		104
-#define			MIXED_SEPARATOR		1000        /* magic number for mixed mode */
-#define			ANSWER_LEN			8
-#define			KANA_SET_NAMES		15
-#define			KANA_MODE_NAMES		3
-#define			MAX_NUMBER_OF_SIGNS (NUMBER_OF_SIGNS*2)
+gchar *     get_test_mode_name      (gint mode);
+gchar *     get_kana_set_name       (gint set);
+gchar *     get_kana_sign           (gint index, gint kana_type);
 
-#define         ANSWER_DELIMITER    ","
-#define         MAX_RANSWERS        10
-
-extern gint		question_counter;
-extern gint		max_entries_in_test;
-
-extern gint		questions_table[];
-extern gchar	*kana_signs[];
-extern gchar    *kana_signs_names[];
-extern gchar    *kana_mode_names[];
-
-void			test_init			    (void);
-void            test_info               (void);
-void			test_generate_tables    (gint number);
-void			test_check_answer	    (gchar *user_answer);
-gchar*			test_sec2str            (guint seconds, gboolean mshort);
+void        test_init               (GUI *appGUI);
+void        test_info               (GUI *appGUI);
+void        test_check_answer       (gchar *user_answer, GUI *appGUI);
+void        test_generate_tables    (gint number_of_questions, GUI *appGUI);
+gchar *     test_sec2str            (guint seconds, gboolean mshort);
 
 #endif /* _TEST_H */
 
